@@ -97,7 +97,9 @@ public:
 
     inline int GetDescriptor(void) { return sd; }
 
-    void SetFilter(const string &expr);
+    inline void SetFilter(const string &expr) {
+        ndCaptureThread::CompileFilter(expr, filter);
+    }
     bool ApplyFilter(const uint8_t *pkt, size_t snaplen,
       size_t length) const;
 
@@ -403,30 +405,6 @@ ndPacketRing::~ndPacketRing() {
     for (auto &i : blocks) delete i;
 }
 
-void ndPacketRing::SetFilter(const string &expr) {
-#ifdef HAVE_PCAP_OPEN_DEAD
-    pcap_t *pcap = pcap_open_dead(DLT_EN10MB, ndGC.max_capture_length);
-    if (pcap == nullptr) {
-        throw ndException(__PRETTY_FUNCTION__,
-          "error creating PCAP context");
-    }
-    if (pcap_compile(pcap, &filter, expr.c_str(), 1,
-          PCAP_NETMASK_UNKNOWN) == -1)
-    {
-#else
-    if (pcap_compile_nopcap(ndGC.max_capture_length, DLT_EN10MB,
-          &filter, expr.c_str(), 1, PCAP_NETMASK_UNKNOWN) == -1)
-    {
-#endif
-        throw ndException(__PRETTY_FUNCTION__,
-          "error compiling BPF filter");
-    }
-
-#ifdef HAVE_PCAP_OPEN_DEAD
-    pcap_close(pcap);
-#endif
-}
-
 bool ndPacketRing::ApplyFilter(const uint8_t *pkt,
   size_t length, size_t snaplen) const {
     return (filter.bf_insns &&
@@ -536,23 +514,20 @@ ndPacket *ndPacketRing::CopyPacket(const void *entry,
         return nullptr;
     }
 
-    ndPacket *pkt = nullptr;
     // One-and-only packet copy...
     uint8_t *pkt_data = new uint8_t[tp_snaplen];
+    memcpy(pkt_data, data, tp_snaplen);
 
-    if (pkt_data) {
-        memcpy(pkt_data, data, tp_snaplen);
-        pkt = new ndPacket(status, tp_len, tp_snaplen, pkt_data, tv);
-    }
+    ndPacket *pkt = new ndPacket(status, tp_len, tp_snaplen,
+      pkt_data, tv);
 
-    if (pkt) status |= ndPacket::StatusFlags::OK;
-    else status = ndPacket::StatusFlags::ALLOC_ERROR;
+    status |= ndPacket::StatusFlags::OK;
 
     return pkt;
 }
 
 ndCaptureTPv3::ndCaptureTPv3(int16_t cpu,
-  nd_iface_ptr &iface, const ndDetectionThreads &threads_dpi,
+  ndInterface::Ptr &iface, const ndDetectionThreads &threads_dpi,
   ndDNSHintCache *dhc, uint8_t private_addr)
   : ndCaptureThread(ndCaptureType::TPV3, cpu, iface,
       threads_dpi, dhc, private_addr),
@@ -576,8 +551,6 @@ void *ndCaptureTPv3::Entry(void) {
 
     ndPacketRing *_ring = new ndPacketRing(iface->ifname,
       iface->config_tpv3, &stats);
-    if (_ring == nullptr)
-        throw runtime_error(strerror(ENOMEM));
 
     ring = static_cast<void *>(_ring);
 
@@ -596,6 +569,13 @@ void *ndCaptureTPv3::Entry(void) {
 
     capture_state = State::ONLINE;
 
+    unsigned ifindex = if_nametoindex(tag.c_str());
+    if (ifindex == 0) {
+        nd_dprintf("%s: if_nametoindex: %s\n", tag.c_str(), strerror(errno));
+        iface->ifindex = -1;
+    }
+    else iface->ifindex = (int)ifindex;
+
     bool warnings = true;
 
     while (! ShouldTerminate() && rc >= 0) {
@@ -642,6 +622,14 @@ void *ndCaptureTPv3::Entry(void) {
             rc = 0;
             warnings = true;
             capture_state = State::ONLINE;
+
+            ifindex = if_nametoindex(tag.c_str());
+            if (ifindex == 0) {
+                nd_dprintf("%s: if_nametoindex: %s\n",
+                    tag.c_str(), strerror(errno));
+                iface->ifindex = -1;
+            }
+            else iface->ifindex = (int)ifindex;
         }
 
         entry->ProcessPackets(_ring, pkt_queue);
